import pandas as pd
import os
import xlrd
import openpyxl
from openpyxl.utils.cell import column_index_from_string  # 엑셀 열 이름을 숫자로 변환
from openpyxl.utils import get_column_letter  # 숫자를 엑셀 열 이름으로 변환
# import xlwt  # xlwt는 .xls 파일을 수정하기 위해 사용합니다.
from xlutils.copy import copy  # xlutils를 사용하여 기존 .xls 파일 복사
import xlwings as xw  # xlwings 사용
import time

start = time.time()
# FILE = "01vlookup.xls"  # .xls 파일 경로
# FILE = "01vlookup.xlsx"  # .xlsx 파일 경로
FILE = "01vlookup.xlsm"  # .xlsm 파일 경로

# df1 설정 ----------
SOURCE_SH = "Sheet1"
COL_START = "A"
COL_END = "B"

# 파일 확장자 확인
file_ext = os.path.splitext(FILE)[1]  # 확장자만 추출 (.xls, .xlsx, .xlsm)

COUNT_SKIPROW = 0  # 초기화

if file_ext == '.xls':
    # .xls 파일 처리 (xlrd 엔진 사용)
    wb = xlrd.open_workbook(FILE)
    ws = wb.sheet_by_name(SOURCE_SH)

    # "이름"이 나올 때까지 카운트
    for row in range(ws.nrows):  # 총 행 수 만큼 반복
        if ws.cell_value(row, 0) == "이름":  # 첫 번째 열에서 "이름" 확인
            break
        COUNT_SKIPROW += 1  # "이름"이 나오기 전까지 카운트

    # DataFrame 생성
    df1 = pd.read_excel(FILE, sheet_name=SOURCE_SH,
                        header=COUNT_SKIPROW,
                        usecols=f"{COL_START}:{COL_END}",
                        engine='xlrd')

elif file_ext == '.xlsx':
    # .xlsx 파일 처리 (openpyxl 엔진 사용)
    FIND_COL = "A"
    FIND_TITLE = "이름"
    COUNT_SKIPROW = 0  # 초기화

    wb = openpyxl.load_workbook(FILE)  # 워크북 객체 생성
    ws = wb[SOURCE_SH]  # 시트 객체 생성
    for row in ws.iter_rows(min_col=column_index_from_string(FIND_COL), max_col=column_index_from_string(FIND_COL)):  # A열만 확인
        if row[0].value == FIND_TITLE:  # "이름"이 있는지 확인
            break
        COUNT_SKIPROW += 1  # "이름"이 나오기 전까지 카운트

    df1 = pd.read_excel(FILE, sheet_name=SOURCE_SH,
                        header=COUNT_SKIPROW,
                        usecols=f"{COL_START}:{COL_END}",
                        engine='openpyxl')

elif file_ext == '.xlsm':
    # .xlsm 파일 처리 (xlwings 엔진 사용)
    app = xw.App(visible=False)  # 숨김 모드에서 Excel을 엽니다.
    wb = app.books.open(FILE)
    ws = wb.sheets[SOURCE_SH]

    # 첫 번째 행에서 "이름"을 찾는 과정
    header_row = 1
    while ws.range(f'A{header_row}').value != "이름":  # A열에서 "이름" 찾기
        header_row += 1

    # A열의 마지막 데이터가 있는 행 찾기
    last_row = ws.range('A4').end('down').row

    # 데이터 가져오기
    df1 = pd.DataFrame(ws.range(f"{COL_START}{header_row}:{COL_END}{last_row}").value)
    df1.columns = df1.iloc[0]  # 첫 번째 행을 헤더로 설정
    df1 = df1[1:]  # 첫 번째 행(헤더)을 DataFrame에서 제거

    # Excel 파일 닫기
    wb.close()
    app.quit()

else:
    raise ValueError("지원하지 않는 파일 형식입니다. .xls, .xlsx, .xlsm 파일을 사용하십시오.")

# skiprow 구하기 - 데이터 없는 행 갯수 - 무시해야 할 행 갯수
FIND_COL = "F"
FIND_TITLE = "제품"
COUNT_SKIPROW = 0  # 초기화

if file_ext == '.xls':
    for row in range(ws.nrows):
        if ws.cell_value(row, 5) == FIND_TITLE:  # F열에서 "제품" 확인 (열 인덱스는 0부터 시작)
            break
        COUNT_SKIPROW += 1

elif file_ext == '.xlsx':
    for row in ws.iter_rows(min_col=column_index_from_string(FIND_COL), max_col=column_index_from_string(FIND_COL)):  # F열만 확인
        if row[0].value == FIND_TITLE:  # "제품"이 있는지 확인
            break
        COUNT_SKIPROW += 1

elif file_ext == '.xlsm':
    app = xw.App(visible=False)  # 숨김 모드에서 Excel을 엽니다.
    wb = app.books.open(FILE)
    ws = wb.sheets[SOURCE_SH]
    while ws.range(f'F{COUNT_SKIPROW + 1}').value != FIND_TITLE:
        COUNT_SKIPROW += 1
    wb.close()
    app.quit()

else:
    raise ValueError("지원하지 않는 파일 형식입니다.")

# df2 데이터 생성 ################################
COL_START = "F"
COL_END = "G"
COL_TITLE_NAME = ['제품', '가격']

if file_ext == '.xls':
    df2 = pd.read_excel(FILE, sheet_name=SOURCE_SH,
                        header=0,
                        names=COL_TITLE_NAME,
                        index_col=None,
                        usecols=f"{COL_START}:{COL_END}",
                        skiprows=COUNT_SKIPROW,
                        engine='xlrd')

elif file_ext == '.xlsx' or file_ext == '.xlsm':
    df2 = pd.read_excel(FILE, sheet_name=SOURCE_SH,
                        header=0,
                        names=COL_TITLE_NAME,
                        index_col=None,
                        usecols=f"{COL_START}:{COL_END}",
                        skiprows=COUNT_SKIPROW,
                        engine='openpyxl')

# df3 데이터 병합 ################################
COL_STANDARD_NAME = "제품"
JOIN_HOW = "left"
df3 = df1.merge(df2, on=COL_STANDARD_NAME, how=JOIN_HOW)

# 가격을 정수형으로 변환
df3['가격'] = df3['가격'].astype(int)

# 특정 범위 지우기 ########################################
COL_START = "J"
COL_END = "L"
START_ROW = 4  # 제목열 타이핑 되어있는 상태 - 데이터 시작행 0,1,2,3,4,5
# START_CELL = f'A{START_ROW}'  # START_ROW를 기반으로 시작 셀 설정

if file_ext == '.xls':
    # xlrd로 파일 열기
    wb = xlrd.open_workbook(FILE, formatting_info=True)  # 포맷 정보를 가져오기
    ws = wb.sheet_by_name(SOURCE_SH)
    
    # 수정할 Workbook 생성
    new_wb = copy(wb)  # 기존 워크북 복사
    new_ws = new_wb.get_sheet(0)  # 첫 번째 시트 가져오기
    
    # 데이터 지우기
    for column_letter in range(column_index_from_string(COL_START), column_index_from_string(COL_END) + 1):
        for row in range(START_ROW, ws.nrows):  # START_ROW부터 끝까지 반복
            new_ws.write(row, column_letter - 1, '')  # 값을 빈 문자열로 설정하여 지우기

    # 수정된 파일 저장
    new_wb.save(FILE)

elif file_ext == '.xlsx' or file_ext == '.xlsm':
    app = xw.App(visible=False)
    wb = app.books.open(FILE)
    ws = wb.sheets[SOURCE_SH]
    
    # START_ROW부터 마지막 데이터 행까지 찾음
    START_CELL = f'A{START_ROW}'  # START_ROW를 기반으로 시작 셀 설정
    last_row = ws.range(START_CELL).end('down').row  # START_ROW부터 마지막 데이터가 있는 행 확인

    # 밑에 데이터가 없는 경우, last_row가 START_ROW와 같을 수 있음
    if last_row == START_ROW:
        print("삭제할 데이터가 없습니다.")
    else:
        # START_ROW + 1 부터 마지막 행까지 데이터를 지움 (제목 행은 제외)
        for column_letter in range(column_index_from_string(COL_START), column_index_from_string(COL_END) + 1):
            column_cells = ws.range(f'{get_column_letter(column_letter)}{START_ROW + 1}:'  # 제목행 다음부터
                                    f'{get_column_letter(column_letter)}{last_row}')
            column_cells.value = None  # 제목행 이후 데이터만 지우기
    
    wb.save()
    wb.close()
    app.quit()

else:
    raise ValueError("지원하지 않는 파일 형식입니다.")

# 최종 파일 저장 특정 범위에 쓰기
if file_ext == '.xls':
    wb = xlrd.open_workbook(FILE, formatting_info=True)  # 포맷 정보를 가져오기
    new_wb = copy(wb)  # 기존 워크북 복사
    new_ws = new_wb.get_sheet(0)  # 첫 번째 시트 가져오기

    # df3의 데이터를 새로운 위치에 쓰기
    for row_idx, row in df3.iterrows():
        for col_idx, value in enumerate(row):
            new_ws.write(START_ROW + row_idx, column_index_from_string(COL_START) - 1 + col_idx, value)

    new_wb.save(FILE)

    # 시간측정
    end = time.time()
    print("xls 경과시간:"f"{end - start:.5f} sec")

elif file_ext == '.xlsx':
    with pd.ExcelWriter(FILE, mode='a', engine='openpyxl', if_sheet_exists='overlay') as w:
    #     df3.to_excel(w, sheet_name='Sheet1', index=False, header=False, startcol=column_letter_to_number(start_column), startrow=start_row) # 제목열 제외하고 데이터 시작행 0,1,2,3,4,5
    #     df3.to_excel(w, sheet_name='Sheet1', index=False, header=False, startcol=ord(start_column) - ord('A'), startrow=start_row) # 제목열 제외하고 데이터 시작행 0,1,2,3,4,5
        df3.to_excel(w, sheet_name=SOURCE_SH, index=False, header=False, startcol=column_index_from_string(COL_START) - column_index_from_string("A"), startrow=START_ROW) # 제목열 제외하고 데이터 시작행 0,1,2,3,4,5
    
    # 시간측정
    end = time.time()
    print("xlsx 경과시간:"f"{end - start:.5f} sec")

elif file_ext == '.xlsm':
    app = xw.App(visible=False)
    wb = app.books.open(FILE)
    ws = wb.sheets[SOURCE_SH]

    # df3의 데이터를 새로운 위치에 쓰기
    for row_idx, row in df3.iterrows():
        ws.range(f"{COL_START}{START_ROW + 1 + row_idx}").value = row.values.tolist()

    wb.save()
    wb.close()
    app.quit()

    # 시간측정
    end = time.time()
    print("xlsm 경과시간:"f"{end - start:.5f} sec")
